/*--------------------------------------------------------------------------*/
/*                                                                          */
/*                                                                          */
/*      ------------         Bit-Bucket Software <no-Inc>                   */
/*      \ 10001101 /         Writers and Distributors of                    */
/*       \ 011110 /          No-Cost<no-tm> Software.                       */
/*        \ 1011 /                                                          */
/*         ------                                                           */
/*                                                                          */
/*  Copyright (C) 1987, 1988, 1989 by Robert Hartman and Vincent Perriello  */
/*                                                                          */
/*                                                                          */
/*              Box Drawing subroutines for BinkleyTerm 2.10                */
/*                                                                          */
/*                                                                          */
/*    For complete  details  of the licensing restrictions, please refer    */
/*    to the License  agreement,  which  is published in its entirety in    */
/*    the MAKEFILE and BT.C, and also contained in the file LICENSE.210.    */
/*                                                                          */
/*    USE  OF THIS FILE IS SUBJECT TO THE  RESTRICTIONS CONTAINED IN THE    */
/*    BINKLEYTERM  LICENSING  AGREEMENT.  IF YOU DO NOT FIND THE TEXT OF    */
/*    THIS  AGREEMENT IN ANY OF THE  AFOREMENTIONED FILES,  OR IF YOU DO    */
/*    NOT HAVE THESE FILES,  YOU SHOULD  IMMEDIATELY CONTACT THE AUTHORS    */
/*    AT THE  ADDRESSES LISTED BELOW.  IN NO EVENT SHOULD YOU PROCEED TO    */
/*    USE   THIS  FILE  WITHOUT  HAVING   ACCEPTED  THE  TERMS  OF   THE    */
/*    BINKLEYTERM  LICENSING AGREEMENT,  OR SUCH OTHER  AGREEMENT AS YOU    */
/*    ARE ABLE TO REACH WITH THE AUTHORS.                                   */
/*                                                                          */
/*                                                                          */
/*    The Authors can be reached at the following addresses:                */
/*                                                                          */
/*    Robert C. Hartman                      Vincent E. Perriello           */
/*    Spark Software                         VEP Software                   */
/*    427-3 Amherst Street                   111 Carroll Street             */
/*    CS2032, Suite 232                      Naugatuck, CT 06770            */
/*    Nashua, NH 03061                                                      */
/*                                                                          */
/*    FidoNet 1:132/101                      FidoNet 1:141/491              */
/*    Data    (603) 888-8179                 Data    (203) 729-7569         */
/*                                                                          */
/*    Please feel free to contact us at any time to share your comments     */
/*    about our software and/or licensing policies.                         */
/*                                                                          */
/*                                                                          */
/*   This module is derived from code developed by Augie Hansen in his      */
/*   book "Proficient C" published by Microsoft Press.  Mr. Hansen was      */
/*   kind enough to give us verbal permission to use his routines, and      */
/*   Bob, Vince and Alan (and all our full screen users) are grateful.      */
/*   If you decide to use this code in some package you are doing, give     */
/*   some thought to going out and buying the book. He deserves that.       */
/*                                                                          */
/*--------------------------------------------------------------------------*/


#include <stdio.h>
#include <dos.h>

#ifdef __TURBOC__
#include <mem.h>
#else
#include <memory.h>
#endif

#include "sbuf.h"
#include "xfer.h"
#include "com.h"
#include "zmodem.h"
#include "keybd.h"
#include "sched.h"
#include "externs.h"
#include "prototyp.h"
#include "vfossil.h"

extern BUFFER Sbuf;
extern CELLP Scrnbuf;
extern int cursor_col;
extern int cursor_row;

char blanks[160];

int sb_show ()
{
   register int r;
   unsigned int src_os;

   if (screen_blank && do_screen_blank)
      {
      for (r = 0; r < SB_COLS; r += 2)
         {
         blanks[r] = ' ';
         blanks[r + 1] = 7;
         }

      for (r = 0; r < SB_ROWS; r++)
         {
         VioWrtCellStr ((char far *)blanks, SB_COLS * 2, r, 0, 0); /*PLF Fri  05-05-1989  05:05:23 */
         }

      sb_dirty ();
      return (SB_OK);
      }

   /* Anything to do? */
   if (!(Sbuf.flags & SB_DELTA))
      {
      return (SB_OK);
      }

   src_os = 0;
   for (r = 0; r < SB_ROWS; r++)
      {
      /* Copy only changed portions of lines */
      if ((Sbuf.lcol[r] < SB_COLS) && (Sbuf.rcol[r] > 0))
         {
         VioWrtCellStr (
            (char far *)(Scrnbuf + src_os + Sbuf.lcol[r]), /*PLF Fri  05-05-1989  05:06:44 */
            (Sbuf.rcol[r] - Sbuf.lcol[r] + 1) * 2,
            r,
            Sbuf.lcol[r],
            0);

         Sbuf.lcol[r] = SB_COLS;
         Sbuf.rcol[r] = 0;
         }
      src_os += SB_COLS;
      }

   /* the display now matches the buffer -- clear flag bit */
   Sbuf.flags &= ~SB_DELTA;

   gotoxy (cursor_col, cursor_row);
   return (SB_OK);
}

/*
 * Just cleans up the structure to say it is reality - I can use this when
 * I write directly to the screen for single char writes.
 */

void sb_clean ()
{
   int r;

   for (r = 0; r < SB_ROWS; r++)
      {
      Sbuf.lcol[r] = SB_COLS;
      Sbuf.rcol[r] = 0;
      }

   Sbuf.flags &= ~SB_DELTA;
}

/*
 * Make the entire buffer "dirty" so it will be updated.
 */

void sb_dirty ()
{
   int r;

   for (r = 0; r < SB_ROWS; r++)
      {
      Sbuf.lcol[r] = 0;
      Sbuf.rcol[r] = SB_COLS - 1;
      }

   Sbuf.flags |= SB_DELTA;
}
