/*--------------------------------------------------------------------------*/
/*                                                                          */
/*                                                                          */
/*      ------------         Bit-Bucket Software <no-Inc>                   */
/*      \ 10001101 /         Writers and Distributors of                    */
/*       \ 011110 /          No-Cost<no-tm> Software.                       */
/*        \ 1011 /                                                          */
/*         ------                                                           */
/*                                                                          */
/*  Copyright (C) 1987, 1988, 1989 by Robert Hartman and Vincent Perriello  */
/*                                                                          */
/*                                                                          */
/*             Scheduler definitions used in BinkleyTerm 2.10               */
/*                                                                          */
/*                                                                          */
/*    For complete  details  of the licensing restrictions, please refer    */
/*    to the License  agreement,  which  is published in its entirety in    */
/*    the MAKEFILE and BT.C, and also contained in the file LICENSE.210.    */
/*                                                                          */
/*    USE  OF THIS FILE IS SUBJECT TO THE  RESTRICTIONS CONTAINED IN THE    */
/*    BINKLEYTERM  LICENSING  AGREEMENT.  IF YOU DO NOT FIND THE TEXT OF    */
/*    THIS  AGREEMENT IN ANY OF THE  AFOREMENTIONED FILES,  OR IF YOU DO    */
/*    NOT HAVE THESE FILES,  YOU SHOULD  IMMEDIATELY CONTACT THE AUTHORS    */
/*    AT THE  ADDRESSES LISTED BELOW.  IN NO EVENT SHOULD YOU PROCEED TO    */
/*    USE   THIS  FILE  WITHOUT  HAVING   ACCEPTED  THE  TERMS  OF   THE    */
/*    BINKLEYTERM  LICENSING AGREEMENT,  OR SUCH OTHER  AGREEMENT AS YOU    */
/*    ARE ABLE TO REACH WITH THE AUTHORS.                                   */
/*                                                                          */
/*                                                                          */
/*    The Authors can be reached at the following addresses:                */
/*                                                                          */
/*    Robert C. Hartman                      Vincent E. Perriello           */
/*    Spark Software                         VEP Software                   */
/*    427-3 Amherst Street                   111 Carroll Street             */
/*    CS2032, Suite 232                      Naugatuck, CT 06770            */
/*    Nashua, NH 03061                                                      */
/*                                                                          */
/*    FidoNet 1:132/101                      FidoNet 1:141/491              */
/*    Data    (603) 888-8179                 Data    (203) 729-7569         */
/*                                                                          */
/*    Please feel free to contact us at any time to share your comments     */
/*    about our software and/or licensing policies.                         */
/*                                                                          */
/*--------------------------------------------------------------------------*/


/* Definitions for day of the week */
#define DAY_SUNDAY      0x01
#define DAY_MONDAY      0x02
#define DAY_TUESDAY     0x04
#define DAY_WEDNESDAY   0x08
#define DAY_THURSDAY    0x10
#define DAY_FRIDAY      0x20
#define DAY_SATURDAY    0x40
#define DAY_UNUSED      0x80

#define DAY_WEEK  (DAY_MONDAY|DAY_TUESDAY|DAY_WEDNESDAY|DAY_THURSDAY|DAY_FRIDAY)
#define DAY_WKEND (DAY_SUNDAY|DAY_SATURDAY)

/* Definitions for matrix behavior */
#define MAT_CM          0x0001
#define MAT_DYNAM       0x0002
#define MAT_BBS         0x0004
#define MAT_NOREQ       0x0008
#define MAT_OUTONLY     0x0010
#define MAT_NOOUT       0x0020
#define MAT_FORCED      0x0040
#define MAT_LOCAL       0x0080
#define MAT_SKIP        0x0100
#define MAT_NOMAIL24    0x0200
#define MAT_NOOUTREQ    0x0400
#define MAT_NOCM        0x0800
#define MAT_RESERVED1   0x1000
#define MAT_RESERVED2   0x2000
#define MAT_RESERVED3   0x4000
#define MAT_RESERVED4   0x8000

/*********************************************************************
* If either of these structures are changed, don't forget to change  *
* the BinkSched string in sched.c, as well as the routines that read *
* and write the schedule file (read_sched, write_sched)!!!           *
*********************************************************************/
typedef struct _event
{
   int days;                                     /* Bit field for which days
                                                  * to execute */
   int minute;                                   /* Minutes after midnight to
                                                  * start event */
   int length;                                   /* Number of minutes event
                                                  * runs */
   int errlevel[5];                              /* Errorlevel exits */
   int last_ran;                                 /* Day of month last ran */
   int behavior;                                 /* Behavior mask */
   int wait_time;                                /* Average number of seconds
                                                  * to wait between dials */
   int with_connect;                             /* Number of calls to make
                                                  * with carrier */
   int no_connect;                               /* Number of calls to make
                                                  * without carrier */
   int node_cost;                                /* Maximum cost node to call
                                                  * during this event */
   char cmd[32];                                 /* Chars to append to
                                                  * packer, aftermail and
                                                  * cleanup */
   char month;                                   /* Month when to do it   */
   char day;                                     /* Day of month to do it */
   int extra[1];                                 /* Extra space for later */
} EVENT;

typedef struct _history
{
   int which_day;                                /* Day number for this
                                                  * record */
   int bbs_calls;                                /* Number of BBS callers */
   int mail_calls;                               /* Number of mail calls */
   int calls_made;                               /* Number of outgoing calls
                                                  * made */
   int connects;                                 /* Number of outbound call
                                                  * successes */
   int files_in;                                 /* Number of files received */
   int files_out;                                /* Number of files sent */
   int last_caller;                              /* Type of last call */
   int last_zone;                                /* Zone number of last call */
   int last_net;                                 /* Net  number of last call */
   int last_node;                                /* Node number of last call */
   int next_zone;                                /* Zone number of next call */
   int next_net;                                 /* Net  number of next call */
   int next_node;                                /* Node number of next call */
   int extra[2];
} HISTORY;
